package org.linotte.greffons.impl;

import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.InetAddress;

import org.linotte.greffons.externe.Greffon;

public class TCPClient extends Greffon {

	private ZipSocket socket;
	private boolean debug = true;

	@Slot
	public boolean connexion(String url, int port) throws GreffonException {
		try {
			if (socket != null) {
				try {
					socket.close();
				} catch (Exception e) {
					if (debug)
						e.printStackTrace();
				}
			}
			//throw new GreffonException("Le socket est déjà ouvert");
			socket = new ZipSocket(url, port);
			return true;
		} catch (Exception e) {
			if (debug)
				e.printStackTrace();
			throw new GreffonException("Impossible de se connecter au serveur : " + e.getMessage());
		}
	}

	@Slot
	public boolean fermer() throws GreffonException {
		verifierSocket();
		try {
			socket.close();
			socket = null;
			return true;
		} catch (Exception e) {
			if (debug)
				e.printStackTrace();
			throw new GreffonException("Impossible de fermer la connexion : " + e.getMessage());
		}
	}

	private void verifierSocket() throws GreffonException {
		if (socket == null)
			throw new GreffonException("Le socket n'est pas ouverte");
	}

	@Slot
	public boolean envoyer(ObjetLinotte acteur) throws GreffonException {
		verifierSocket();
		try {
			OutputStream os = socket.getOutputStream();
			ObjectOutputStream oos = new ObjectOutputStream(os);
			oos.writeObject(acteur);
			os.flush();
			return true;
		} catch (Exception e) {
			if (debug)
				e.printStackTrace();
			throw new GreffonException("Impossible d'envoyer l'acteur : " + e.getMessage());
		}
	}

	@Slot
	public ObjetLinotte recevoir() throws GreffonException {
		verifierSocket();
		try {
			ObjectInputStream oos = new ObjectInputStream(socket.getInputStream());
			ObjetLinotte objetLinotte = (ObjetLinotte) oos.readObject();
			return objetLinotte;
		} catch (Exception e) {
			if (debug)
				e.printStackTrace();
			throw new GreffonException("Impossible de recevoir l'acteur : " + e.getMessage());
		}
	}

	@Slot
	public String nommachine() throws GreffonException {
		try {
			return InetAddress.getLocalHost().getHostName();
		} catch (Exception e) {
			if (debug)
				e.printStackTrace();
			return "localhost";
		}
	}

	@Slot(nom = "débogage")
	public boolean debogage(boolean debog) throws GreffonException {
		debug = debog;
		return false;
	}
}
