package org.linotte.greffons.impl.xyz;

import java.io.File;
import java.lang.reflect.Field;

import org.linotte.moteur.outils.Ressources;
import org.linotte.moteur.xml.Version;

/**
 * http://java.developpez.com/faq/java/?page=langage_jni
 * @author cpc
 *
 */
public final class Intialisateur3D {

	private Intialisateur3D() {

	}

	public static void init() {
		try {

			String osname = System.getProperty("os.name").toLowerCase();
			String osarch = System.getProperty("os.arch");

			String path = "3D" + File.separator + ((osname.indexOf("windows") != -1) ? "windows" : "linux");
			path += File.separator + ((osarch.indexOf("86") != -1) ? "x86" : "amd64");

			File libPath = new File(Ressources.getGreffons(), path);//new File("C:/RMO/perso/langagelinotte/greffons/3D/" + path);
			if (Version.isBeta()) System.out.println(libPath.getCanonicalPath());
			addToJavaLibraryPath(libPath);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Ajoute un nouveau répertoire dans le java.library.path.
	 * @param dir Le nouveau répertoire à ajouter.
	 */
	public static void addToJavaLibraryPath(File dir) {
		final String LIBRARY_PATH = "java.library.path";
		if (!dir.isDirectory()) {
			throw new IllegalArgumentException(dir + " is not a directory.");
		}
		String javaLibraryPath = System.getProperty(LIBRARY_PATH);
		System.setProperty(LIBRARY_PATH, javaLibraryPath + File.pathSeparatorChar + dir.getAbsolutePath());

		resetJavaLibraryPath();
	}

	/**
	 * Supprime le cache du "java.library.path".
	 * Cela forcera le classloader à revérifier sa valeur lors du prochaine chargement de librairie.
	 * 
	 * Attention : ceci est spécifique à la JVM de Sun et pourrait ne pas fonctionner
	 * sur une autre JVM...
	 */
	public static void resetJavaLibraryPath() {
		synchronized (Runtime.getRuntime()) {
			try {
				Field field = ClassLoader.class.getDeclaredField("usr_paths");
				field.setAccessible(true);
				field.set(null, null);

				field = ClassLoader.class.getDeclaredField("sys_paths");
				field.setAccessible(true);
				field.set(null, null);
			} catch (NoSuchFieldException e) {
				throw new RuntimeException(e);
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			}
		}
	}

}
