/***********************************************************************
 * Linotte                                                             *
 * Version release date : January 11, 2006                             *
 * Author : Mounes Ronan ronan.mounes@amstrad.eu                       *
 *                                                                     *
 *     http://langagelinotte.free.fr                                   *
 *                                                                     *
 * This code is released under the GNU GPL license, version 2 or       *
 * later, for educational and non-commercial purposes only.            *
 * If any part of the code is to be included in a commercial           *
 * software, please contact us first for a clearance at                *
 *   ronan.mounes@amstrad.eu                                           *
 *                                                                     *
 *   This notice must remain intact in all copies of this code.        *
 *   This code is distributed WITHOUT ANY WARRANTY OF ANY KIND.        *
 *   The GNU GPL license can be found at :                             *
 *           http://www.gnu.org/copyleft/gpl.html                      *
 *                                                                     *
 ***********************************************************************/

package org.linotte.moteur.xml;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.linotte.frame.gui.SplashWindow;
import org.linotte.frame.latoile.Toile;
import org.linotte.greffons.GreffonsHandler;
import org.linotte.moteur.entites.Prototype;
import org.linotte.moteur.entites.PrototypeGraphique;
import org.linotte.moteur.outils.Preference;
import org.linotte.moteur.outils.Ressources;
import org.linotte.moteur.xml.analyse.Grammaire;
import org.linotte.moteur.xml.api.IHM;
import org.linotte.moteur.xml.api.Librairie;

import com.dmurph.tracking.AnalyticsConfigData;
import com.dmurph.tracking.JGoogleAnalyticsTracker;

/**
 * Cette classe doit être initialisée qu'une seule fois. Elle contient la
 * grammaire du langage et la structure des espèces
 * 
 * @author CPC
 *
 */
public class Linotte {

    private Grammaire grammaire = null;

    /**
     * Compatible Linotte 1.7.1
     */
    private boolean syntaxeLinotte1 = false;

    private Librairie<?> lib = null;

    private IHM ihm = null;

    private RegistreDesActions registreDesEtats;

    private JGoogleAnalyticsTracker tracker = null;

    public List<Prototype> especeModeleMap = new ArrayList<Prototype>();

    private boolean forceFurtif = false;

    /**
     * Constructeur utilisé par : Webonotte
     * 
     * @param plib
     */
    public Linotte(Librairie<?> plib) {
	lib = plib;
	forceFurtif = true;
	initVM();
    }

    public Linotte(Librairie<?> plib, IHM pihm, boolean old) {
	lib = plib;
	ihm = pihm;
	syntaxeLinotte1 = old;
	initVM();
    }

    private void initVM() {
	// On utilise le proxy du système :
	// http://www.rgagnon.com/javadetails/java-0085.html
	try {
	    System.setProperty("java.net.useSystemProxies", "true");
	} catch (Exception e) {
	    System.out.println("Pas de proxy, en mode bac à sable");
	}

	InputStream is;
	if (syntaxeLinotte1) {
	    is = Ressources.getFlux("grammaire_fr.xml");
	} else {
	    is = Ressources.getFlux("stricte_grammaire_fr.xml");
	}
	grammaire = new Grammaire(is);
	registreDesEtats = new RegistreDesActions();
	// Initialisation des objets graphiques :
	SplashWindow.setProgressValue("Assemblage des prototypes");
	try {
	    // Enregistrement des especes graphiques sur la toile :
	    PrototypeGraphique.init();
	    // Especes et greffons internes :
	    Bibliotheque.retourneEspeces(lib, this);
	    // Greffons charges depuis l'exterieur :
	    GreffonsHandler.chargerGreffons(Ressources.getGreffons());
	    // initialisation des greffons :
	    Bibliotheque.initGreffons(lib, this);
	} catch (Exception e) {
	    e.printStackTrace();
	}

	initStatistique();
    }

    private void initStatistique() {
	if (!Toile.isApplet() && !forceFurtif && Preference.getIntance().get(Preference.P_MODE_FURTIF) == null) {
	    AnalyticsConfigData config = new AnalyticsConfigData("UA-44967742-1");
	    config.populateFromSystem();
	    tracker = new JGoogleAnalyticsTracker(config, JGoogleAnalyticsTracker.GoogleAnalyticsVersion.V_4_7_2);
	    tracker.trackPageView("/start/" + Version.getVersion() + "/" + (syntaxeLinotte1 ? "version1" : "version2"), "Atelier " + Version.getVersion() + " lancé", null);
	} else {
	    RegistreDesActions.ajouterErreur("Big brother isn't watching you !");
	}
    }

    public Librairie<?> getLibrairie() {
	return lib;
    }

    public Grammaire getGrammaire() {
	return grammaire;
    }

    public RegistreDesActions getRegistreDesEtats() {
	return registreDesEtats;
    }

    public void setIhm(IHM ihm) {
	this.ihm = ihm;
    }

    public IHM getIhm() {
	return ihm;
    }

    public boolean isSyntaxeLinotte1() {
	return syntaxeLinotte1;
    }

}