/***********************************************************************
 * Linotte                                                             *
 * Version release date : September 01, 2006                           *
 * Author : Mounes Ronan ronan.mounes@amstrad.eu                       *
 *                                                                     *
 *     http://langagelinotte.free.fr                                   *
 *                                                                     *
 * This code is released under the GNU GPL license, version 2 or       *
 * later, for educational and non-commercial purposes only.            *
 * If any part of the code is to be included in a commercial           *
 * software, please contact us first for a clearance at                *
 *   ronan.mounes@amstrad.eu                                           *
 *                                                                     *
 *   This notice must remain intact in all copies of this code.        *
 *   This code is distributed WITHOUT ANY WARRANTY OF ANY KIND.        *
 *   The GNU GPL license can be found at :                             *
 *           http://www.gnu.org/copyleft/gpl.html                      *
 *                                                                     *
 ***********************************************************************/

package org.linotte.moteur.xml;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.linotte.moteur.xml.actions.*;
import org.linotte.moteur.xml.alize.kernel.Action;

public class RegistreDesActions {

	public RegistreDesActions() {
		initEtatsDuSysteme();
	}

	private Map<String, Action> map = null;

	private static Set<String> erreurs = new HashSet<String>();

	private EtatTemporiser temporiser;
	private EtatPause pause;
	private EtatAttendreSeconde attendreSeconde;
	private EtatAttendreMilliSeconde attendreMilliSeconde;
	private EtatAppeler etatAppeler;
	private EtatCondition etatCondition;

	public void initEtatsDuSysteme() {

		if (map == null) {
			map = new HashMap<String, Action>();
			addAction(new EtatLivre());
			addAction(new EtatActeur());
			addAction(new EtatEspece());
			addAction(new EtatRole());
			addAction(new EtatParagraphe());
			addAction(new EtatAfficher());
			addAction(new EtatDemander());
			addAction(new EtatCopier());
			addAction(new EtatMesurer());
			addAction(new EtatAjouter());
			addAction(new EtatAction());
			addAction(new EtatEffacerLeTableau());
			addAction(new EtatQuestionner());
			addAction(new EtatArreter());
			addAction(new EtatPause());
			addAction(new EtatTemporiser());
			addAction(new EtatAttendreSeconde());
			addAction(new EtatAttendreMilliSeconde());
			addAction(new EtatMultiplier());
			addAction(new EtatSoustraire());
			addAction(new EtatMelanger());
			addAction(new EtatTrier());
			addAction(new EtatVider());
			addAction(new EtatLire());
			addAction(new EtatAller());
			addAction(new EtatFaire());
			addAction(new EtatRevenir());
			addAction(new EtatBoucle());
			addAction(etatCondition = new EtatCondition());
			addAction(new EtatConditionSinon());
			addAction(new EtatDiviser());
			addAction(new EtatInverser());
			addAction(new EtatExtraire());
			addAction(new EtatInserer());
			addAction(new EtatChercher());
			addAction(new EtatSon());
			addAction(new EtatConcatener());
			addAction(new EtatValoir());
			addAction(new EtatProjeter());
			addAction(new EtatDeplacerVers());
			addAction(new EtatEffacerLaToile());
			addAction(new EtatEffacer());
			addAction(new EtatDeplacerDe());
			addAction(new EtatDeplacerVersGauche());
			addAction(new EtatDeplacerVersDroite());
			addAction(new EtatDeplacerVersHaut());
			addAction(new EtatDeplacerVersBas());
			addAction(new EtatViderTouches());
			addAction(new EtatPhotographier());
			addAction(new EtatStimuler());
			addAction(new EtatConvertir());
			addAction(new EtatFermer());
			addAction(new EtatExplorer());
			addAction(new EtatParcourir());
			addAction(new EtatImportLivre());
			addAction(new EtatParcourirDe());
			addAction(new EtatOter());
			addAction(new EtatModifier());
			addAction(new EtatRafraichir());
			addAction(new EtatAvancerDe());
			addAction(new EtatTournerADroite());
			addAction(new EtatTournerAGauche());
			addAction(new EtatReculerDe());
			addAction(new EtatAppeler());
			addAction(new EtatObserver());
			addAction(new EtatOuvrir());
			addAction(new EtatFermerTube());
			addAction(new EtatCharger());
			addAction(new EtatDecharger());
			addAction(new EtatConfigurer());
			addAction(new EtatEvaluer());
			addAction(new EtatSouffleur());
			addAction(new EtatFusionner());
			addAction(new EtatFaireReagir());
			addAction(new EtatRetourner());
			addAction(new EtatEssayer());
			addAction(new EtatPeindre());
			addAction(new EtatPlusFaireReagir());
			addAction(new EtatImportation());
			addAction(new EtatAnnihiler());
			addAction(new EtatPiquer());
			addAction(new EtatAttacher());
			addAction(new EtatEvoquer());
			addAction(new EtatDecrementer());
			addAction(new EtatIncrementer());
			addAction(new EtatTilde());
			addAction(new EtatTestUnitaireIn());
			addAction(new EtatTestUnitaireOut());
			addAction(new EtatTestUnitaire());
			addAction(new EtatCompatibilite());
			addAction(new EtatStopper());
			addAction(new EtatMontrer());
			addAction(new EtatProposer());
			addAction(new EtatStructureGlobale());
			addAction(new EtatStructureDebut());
			addAction(new EtatRecharger());
			addAction(new EtatGreffons());
			addAction(new EtatInterrompre());
		}
	}

	public void addAction(Action etat) {
		if (etat instanceof EtatAttendreSeconde) {
			attendreSeconde = (EtatAttendreSeconde) etat;
		} else if (etat instanceof EtatAttendreMilliSeconde) {
			attendreMilliSeconde = (EtatAttendreMilliSeconde) etat;
		} else if (etat instanceof EtatTemporiser) {
			temporiser = (EtatTemporiser) etat;
		} else if (etat instanceof EtatPause) {
			pause = (EtatPause) etat;
		} else if (etat instanceof EtatAppeler) {
			etatAppeler = (EtatAppeler) etat;
		}
		map.put(etat.clef(), etat);
	}

	public Action retourneAction(String clef) {
		return map.get(clef);
	}

	public static void ajouterErreur(String message) {
		erreurs.add(message);
	}

	public static Iterator<String> retourneErreurs() {
		Iterator<String> i = erreurs.iterator();
		return i;
	}

	public EtatTemporiser getTemporiser() {
		return temporiser;
	}

	public EtatPause getPause() {
		return pause;
	}

	public EtatAttendreSeconde getAttendreSeconde() {
		return attendreSeconde;
	}

	public EtatAttendreMilliSeconde getAttendreMilliSeconde() {
		return attendreMilliSeconde;
	}

	public EtatAppeler getActionAppeler() {
		return etatAppeler;
	}

	public EtatCondition getActionCondition() {
		return etatCondition;
	}

}
